#!/bin/bash
#
# Copyright (C) 2019 OX Software GmbH
# 
# This file is part of OX Automation.
#
# OX Automation is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# OX Automation is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# 
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with OX Automation. If not, see <http://www.gnu.org/licenses/>.
#
#
# Authors:
# Benedikt Kroening <benedikt.kroening@open-xchange.com>
#

set -e
SCRIPT_DIR="$( cd "$(dirname "$0")" ; pwd -P)"
# sourcing some commonly used functions and executing basic checks
source ${SCRIPT_DIR}/../lib/common.bash
source ${SCRIPT_DIR}/../lib/setup.bash
checkRootPrivileges
detectDistribution

## Help text
show_usage() {
echo -n "
  $0 [OPTIONS]


"
}


### reading config from command line; overriding defaults
POSITIONAL=()
while [[ $# -gt 0 ]]
do
key="$1"

case $key in
    -h|--help)
    show_usage
    exit 0
    shift
    ;;
    --debug)
    echo "WARNING: Debug flag set (--debug)"
    set -x
    shift
    ;;
    -db|--database)
    DATABASE_TYPE="$2"
    shift
    shift
    ;;
    # --repo-guard)
    # OX_REPO_GUARD="$2"
    # echo "WARNING: Using $OX_REPO_GUARD as guard repository source"
    # shift 
    # shift 
    # ;;
    *)    # unknown option
    POSITIONAL+=("$1") # save it in an array for later
    shift 
    ;;
esac
done
set -- "${POSITIONAL[@]}" # restore positional parameters

if [[ "$DATABASE_TYPE" == "mysql" ]] ; then
    echo "MySQL database type not yet implemented"
    exit 1
fi

if [[ "$DATABASE_TYPE" == "mariadb" ]] ; then
    echo "Installing mariadb database server"
    installPackages "mariadb-server"
fi



    # These steps are not required if this setup is docker


if [[ "${INSTALL_TYPE}" != "docker" ]]; then
# Post-install actions
    if [[ "${DIST}" == "DebianJessie" ]] || [[ "${DIST}" == "DebianStretch" ]]; then
        echo "Nothing to do here ..."
    elif [[ "${DIST}" == "SLES12" ]]; then
        systemctl start mysql.service
        chkconfig mysql on

        if [[ -n "${DATABASE_ROOT_PASS}" ]]; then
            echo "Setting MySQL root password..."
            /usr/bin/mysqladmin -u root password ${DATABASE_ROOT_PASS}
        fi
    elif [[ true == "${ANY_CENTOS}" ]] || [[ true == "${ANY_RHEL}" ]]; then
        if [[ "${DIST}" == "CENTOS6" ]] || [[ "${DIST}" == "RHEL6" ]]; then
            if [[ "${DATABASE_TYPE}" == "mariadb" ]] ; then
                service mysql start
                chkconfig --level 345 mysql on
            else
                service mysqld start
                chkconfig --level 345 mysqld on
            fi
        elif [[ "${DIST}" == "CENTOS7" ]] || [[ "${DIST}" == "RHEL7" ]]; then
            systemctl start mariadb.service
            systemctl enable mariadb.service
        fi

        if [[ -n "${DATABASE_ROOT_PASS}" ]]; then
            echo "Setting MySQL root password..."
            /usr/bin/mysqladmin -u root password "${DATABASE_ROOT_PASS}"
        fi
    fi
fi