#!/bin/bash
#
# Copyright (C) 2019 OX Software GmbH
# 
# This file is part of OX Automation.
#
# OX Automation is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# OX Automation is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# 
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with OX Automation. If not, see <http://www.gnu.org/licenses/>.
#
#
# Authors:
# Benedikt Kroening <benedikt.kroening@open-xchange.com>
#
set -e
SCRIPT_DIR="$( cd "$(dirname "$0")" ; pwd -P)"
# sourcing some commonly used functions and executing basic checks
source ${SCRIPT_DIR}/../lib/common.bash
source ${SCRIPT_DIR}/../lib/setup.bash

INSTALL_GUARD_BACKEND=false
INSTALL_GUARD_MIDDLEWARE=false
INSTALL_GUARD_FRONTEND=false

## Help text
show_usage() {
echo -n "
    --backend       - This will install a guard backend node (for cluster use)
    --middleware    - This will install guard api and ui manifest
    --frontend      - This will install the guard frontend packages

"
}

### reading config from command line; overriding defaults
POSITIONAL=()
while [[ $# -gt 0 ]]
do
key="$1"

case $key in
    -h|--help)
    show_usage
    exit 0
    shift
    ;;
    --debug)
    echo "WARNING: Debug flag set (--debug)"
    set -x
    shift
    ;;
    --backend) # non value argument
    INSTALL_GUARD_BACKEND=true
    shift
    ;;
    --middleware) # non value argument
    INSTALL_GUARD_MIDDLEWARE=true
    shift
    ;;
    --frontend) # non value argument
    INSTALL_GUARD_FRONTEND=true
    shift
    ;;
    *) # unknown option
    echo "Unknonwn option: $1"
    POSITIONAL+=("$1") # save it in an array for later
    shift 
    ;;
esac
done
set -- "${POSITIONAL[@]}" # restore positional parameters


if [[ -z "${AS_GUARD_VERSION}" ]] || [[ "${INSTALL_GUARD}" = false  ]]; then
    echo ""
    echo "WARNING: Skipping guard setup"
    echo ""
    sleep 3
    exit 0
fi

# 
if [[ "${INSTALL_GUARD_MIDDLEWARE}" = true  ]]; then
    if [[ "${INSTALL_GUARD_BACKEND}" = true  ]] || [[ "${INSTALL_TYPE}" == "singlenode" ]]; then
        # These packages are guard only, see below for middleware realted guard packages
        PACKAGES="open-xchange-guard open-xchange-guard-file-storage"
    fi

    PACKAGES="${PACKAGES} open-xchange-guard-ui open-xchange-guard-backend-plugin"
fi

if [[ "${INSTALL_GUARD_FRONTEND}" = true  ]]; then
    PACKAGES="${PACKAGES} open-xchange-guard-ui-static open-xchange-guard-reader open-xchange-guard-help*"
fi

installPackages ${PACKAGES}
